
/* $XConsortium: sunFbs.c,v 1.8 94/08/16 13:45:30 dpw Exp $ */

/*
Copyright (c) 1990, 1993  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.
 */

/************************************************************
Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.

                    All Rights Reserved

Permission  to  use,  copy,  modify,  and  distribute   this
software  and  its documentation for any purpose and without
fee is hereby granted, provided that the above copyright no-
tice  appear  in all copies and that both that copyright no-
tice and this permission notice appear in  supporting  docu-
mentation,  and  that the names of Sun or X Consortium
not be used in advertising or publicity pertaining to 
distribution  of  the software  without specific prior 
written permission. Sun and X Consortium make no 
representations about the suitability of this software for 
any purpose. It is provided "as is" without any express or 
implied warranty.

SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
THE USE OR PERFORMANCE OF THIS SOFTWARE.

********************************************************/

/*
 * Copyright (c) 1987 by the Regents of the University of California
 * Copyright (c) 1987 by Adam de Boor, UC Berkeley
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

/****************************************************************/
/* Modified from  sunCG4C.c for X11R3 by Tom Jarmolowski	*/
/****************************************************************/

/************************************************************
Copyright (c) 1994  Sun Microsystems, Inc.
Copyright (c) 1993  X Consortium

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documenta-
tion files (the "Software"), to deal in the Software with-
out restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to
whom the Software is furnished to do so, subject to the 
following conditions:

The above copyright notice and this permission notice shall
be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE, AND NONINFRINGEMENT.  IN NO EVENT SHALL SUN OR THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHER-
WISE, ARISING FROM, OUT OF, OR IN CONNECTION WITH THE 
SOFTWARE OR THE USE OR DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the names of Sun
Microsystems, Inc. and the X Consortium shall not be used in
advertising or otherwise to promote the sale, use, or other
dealings in this Software without prior written authoriza-
tion from Sun Microsystems, Inc. and/or the X Consortium, as
appropriate.
*************************************************************/

/*
 * modified for X11R6 Xvan server
 */

#include "sun.h"
#include <sys/mman.h>

int sunScreenIndex;

static unsigned long generation = 0;

pointer sunMemoryMap (size_t len, off_t off, int fd)
{
    int		pagemask, mapsize;
    caddr_t	addr;
    pointer	mapaddr;

#ifdef SVR4
    pagemask = sysconf(_SC_PAGESIZE) - 1;
#else 
    pagemask = getpagesize() - 1;
#endif
    mapsize = ((int) len + pagemask) & ~pagemask;
    addr = 0;

#ifndef _MAP_NEW
    if ((addr = (caddr_t) valloc (mapsize)) == NULL) {
	Error ("sunKMemoryMap(): Couldn't allocate frame buffer memory");
	(void) close (fd);
	return NULL;
    }
#endif

    /* 
     * try and make it private first, that way once we get it, an
     * interloper, e.g. another server, can't get this frame buffer,
     * and if another server already has it, this one won't.
     */
    if ((int)(mapaddr = (pointer) mmap (addr,
		mapsize,
		PROT_READ | PROT_WRITE, MAP_PRIVATE,
		fd, off)) == -1)
	mapaddr = (pointer) mmap (addr,
		    mapsize,
		    PROT_READ | PROT_WRITE, MAP_SHARED,
		    fd, off);
    if (mapaddr == (pointer) -1) {
	Error ("sunMemoryMap(): mapping frame buffer memory");
	(void) close (fd);
	mapaddr = (pointer) NULL;
    }
    return mapaddr;
}

extern "C" int AllocateScreenPrivateIndex();

Bool sunScreenAllocate (ScreenPtr pScreen)
{
    sunScreenPtr    pPrivate;

    if (generation != serverGeneration)
    {
	sunScreenIndex = AllocateScreenPrivateIndex();
	if (sunScreenIndex < 0)
	    return FALSE;
	generation = serverGeneration;
    }
    pPrivate = (sunScreenPtr) xalloc (sizeof (sunScreenRec));
    if (!pPrivate)
	return FALSE;

    pScreen->devPrivates[sunScreenIndex].ptr = (pointer) pPrivate;
    return TRUE;
}

Bool sunSaveScreen (ScreenPtr pScreen, int on)
{
  int state;
  fbFd *pfbfd;

  if (on != SCREEN_SAVER_FORCER) {
    if (on == SCREEN_SAVER_ON)
      state = 0;
    else
      state = 1;
    for (pfbfd = sunFbs[pScreen->myNum]; pfbfd; pfbfd = pfbfd->next)
      (void) ioctl(pfbfd->fd, FBIOSVIDEO, &state);
  }
  return( TRUE );
}

static Bool closeScreen (int i, ScreenPtr pScreen)
{
    SetupScreen(pScreen);
    Bool    ret;

    (void) OsSignal (SIGIO, SIG_IGN);
    sunDisableCursor (pScreen);
    pScreen->CloseScreen = pPrivate->CloseScreen;
    ret = (*pScreen->CloseScreen) (i, pScreen);
    (void) (*pScreen->SaveScreen) (pScreen, SCREEN_SAVER_OFF);
    xfree ((pointer) pPrivate);
    return ret;
}

Bool sunScreenInit (ScreenPtr pScreen)
{
    SetupScreen(pScreen);
    extern void sunBlockHandler(int, pointer, struct timeval **, pointer);
    extern void sunWakeupHandler(int, pointer, unsigned long, pointer);
    static ScreenPtr autoRepeatScreen;
    extern miPointerScreenFuncRec   sunPointerScreenFuncs;

    pPrivate->installedMap = 0;
    pPrivate->CloseScreen = pScreen->CloseScreen;
    pScreen->CloseScreen = closeScreen;
    pScreen->SaveScreen = sunSaveScreen;
#ifdef XKB
    if (noXkbExtension) {
#endif
    /*
     *	Block/Unblock handlers
     */
    if (sunAutoRepeatHandlersInstalled == FALSE) {
	autoRepeatScreen = pScreen;
	sunAutoRepeatHandlersInstalled = TRUE;
    }

    if (pScreen == autoRepeatScreen) {
        pScreen->BlockHandler = sunBlockHandler;
        pScreen->WakeupHandler = sunWakeupHandler;
    }
#ifdef XKB
    }
#endif
    if (!sunCursorInitialize (pScreen))
	miDCInitialize (pScreen, &sunPointerScreenFuncs);
    return TRUE;
}

Bool sunInitCommon (int scrn,
		    ScreenPtr pScrn,
		    off_t offset,
		    Bool (*init1)(ScreenPtr, FBList *, 
				  int, int, int, int, int, int),
		    void  (*init2)(ScreenPtr),
		    Bool (*cr_cm)(ScreenPtr),
		    Bool (*save)(ScreenPtr, int),
		    int	fb_off)
{
  unsigned char *fb;
  FBList *fblist = &sunFBLists[scrn];
  fbFd *pfbfd;

  if (!sunScreenAllocate (pScrn))
    return FALSE;

  for (pfbfd = sunFbs[scrn]; pfbfd; pfbfd = pfbfd->next) {
    if (!pfbfd->fb) {
      if ((fb = (unsigned char *)
	   sunMemoryMap ((size_t) pfbfd->info.fb_size, 
			 offset, 
			 pfbfd->fd)) == NULL)
	return FALSE;
      pfbfd->fb = fb;
    }
    /* ALERT: the following is a hack into private data because I haven't
     * come up with an interface to handle these needs yet... */
    pfbfd->fbdesc = &fblist->fbdesc[fblist->num];
    fblist->addRel(pfbfd->info.fb_width, pfbfd->info.fb_height,
		   pfbfd->info.fb_depth,
		   (unsigned int *) pfbfd->fb,
		   pfbfd->reltolast);
  }

  /* mfbScreenInit() or cfbScreenInit() */
  if (!(*init1)(pScrn, fblist,
		fblist->getWidth(),
		fblist->getHeight(),
		monitorResolution, monitorResolution,
		fblist->getWidth(),
		fblist->getDepth()))
    return FALSE;
  /* sunCGScreenInit() if cfb... */
  if (init2)
    (*init2)(pScrn);
  if (!sunScreenInit(pScrn))
    return FALSE;
  (void) (*save) (pScrn, SCREEN_SAVER_OFF);
  return (*cr_cm)(pScrn);
}

